/*
 *
 * Linup - Uptimes Project Client for Linux
 * Copyright (C) 2002  Joao Sena Ribeiro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "linup.h"
#include <sys/utsname.h>
#include <sys/sysinfo.h>
#include <limits.h>

/* Number to divide the number we get from sysinfo(), regarding *
 * the load average. In my system, it's 65535 (corresponding to *
 * USHRT_MAX).                                                  */
#define LOADDIV USHRT_MAX

/* get_most_info(char*, long int*, float*, char*, char*) -> (void)
 *
 * Gets (through uname() and sysinfo()) the most information
 * possible (idle time is not available through these system
 * calls), and stores it in the pointers it receives).
 *
 */
void get_most_info(char *progname, long *uptime, float *load, char *os,
    char *oslevel) {

  struct sysinfo data;
  struct utsname name;

  if(sysinfo(&data) == -1 || uname(&name) == -1)
    crash(progname, "sysinfo() or uname()");

  *uptime = data.uptime / 60;
  *load = (float) data.loads[0] / (float) LOADDIV;
  strncpy(os, name.sysname, (MAXINFOLENGTH - 1));
  os[(MAXINFOLENGTH - 1)] = '\0';
  strncpy(oslevel, name.release, (MAXINFOLENGTH - 1));
  oslevel[(MAXINFOLENGTH - 1)] = '\0';
}

/* get_cpu(char*, char*) -> (void)
 *
 * Gets (reading a file in the /proc FS) the CPU name. Stores it
 * in the pointer it receives.
 *
 * Much better that getting this from uname(), because it doesn't
 * work right in PPC machines, and it works better like this on
 * x86 machines.
 *
 * Thanks to Carsten Klapp <carstenklapp@users.sourceforge.net>
 * for telling me about this, and suggesting an alternative.
 *
 */
void get_cpu(char *progname, char *cpu) {
  FILE *fp;
  char c, buff[BUFFSIZE], buff2[BUFFSIZE], *cpu_str, *mach_str;
  int i, use_machine = 0;

  if((fp = fopen(PROC_CPUINFO, "r")) == NULL)
    crash(progname, PROC_CPUINFO);

  for(i = 0; ((c = fgetc(fp)) != EOF) && (i < (sizeof(buff) - 1)); i++)
    buff[i] = c;
  buff[i] = '\0';

  strncpy(buff2, buff, (sizeof(buff2) - 1));

  /* Intel */
  cpu_str = strstr(buff, "model name\t: ");
  if(cpu_str != NULL)
    cpu_str += 13;
  use_machine = 1;
  /* Alpha */
  if(cpu_str == NULL) {
    cpu_str = strstr(buff, "platform string\t\t: ");
    if(cpu_str != NULL)
      cpu_str += 19;
	use_machine = 0;
  }
  /* StrongARM */
  if(cpu_str == NULL) {
    cpu_str = strstr(buff, "Processor\t: ");
    if(cpu_str != NULL)
      cpu_str += 12;
  	use_machine = 0;
  }
  /* Sparc */
  if(cpu_str == NULL) {
    cpu_str = strstr(buff, "cpu\t\t: ");
    if(cpu_str != NULL)
      cpu_str += 7;
	use_machine = 0;
  }
  /* PPC */
  if(cpu_str == NULL) {
    cpu_str = strstr(buff, "cpu\t\t: ");
    if(cpu_str != NULL)
      cpu_str += 7;
	use_machine = 0;
  }

  if(cpu_str != NULL) {
    for(i = 0; cpu_str[i] != '\n'; i++)
      ;
    cpu_str[i] = '\0';
	
	if(use_machine == 1) {
      /* Intel */
      mach_str = strstr(buff2, "vendor_id\t: ");
	  if(mach_str != NULL)
        mach_str += 12;

	  if(mach_str == NULL)
        mach_str[0] = '\0';

      for(i = 0; mach_str[i] != '\n'; i++)
        ;
      mach_str[i] = '\0';

      snprintf(cpu, BUFFSIZE, "%s %s", mach_str, cpu_str);
	} else
      snprintf(cpu, BUFFSIZE, "%s", cpu_str);
  } else {
	/* If /proc/cpuinfo is not in the expected format (if, for    *
     * example, the machine is not one of the above), fallback to *
     * getting the CPU info from uname().                         */
    struct utsname name;

    if(uname(&name) == -1)
      crash(progname, "uname()");

	strncpy(cpu, name.machine, (BUFFSIZE - 1));
  }

  fclose(fp);
}

/* get_idle(char*, int*) -> (void)
 *
 * Gets (reading a file in the /proc FS) the idle time
 * information. Stores it in the pointer it receives.
 *
 * Idle time information is not available through uname() or
 * sysinfo(), so we need a separate function for this.
 *
 */
void get_idle(char *progname, int *idle) {
  FILE *fp;
  char c, buff[BUFFSIZE];
  int i;
  long int uptime;
  double cpu_idle;

  if((fp = fopen(PROC_UPTIME, "r")) == NULL)
    crash(progname, PROC_UPTIME);

  for(i = 0; ((c = fgetc(fp)) != EOF) && (i < (sizeof(buff) - 1)); i++)
    buff[i] = c;
  buff[i] = '\0';

  sscanf(buff, "%ld.%*d %lf", &uptime, &cpu_idle);

  /* The idle formula is:   *
   *   (idle/uptime)*100    *
   * everything in seconds. */
  *idle = (int) ((cpu_idle / uptime) * 100.0);

  fclose(fp);
}
